package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProjectSort;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProjectSortRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 质量巡检项目分类导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class ProjectSortExcelService implements IExcelImportExtService {

    @Autowired
    private MesProjectSortRepository projectSortRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 质量巡检项目分类导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesProjectSort(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesProjectSort(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 质量巡检项目分类导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesProjectSort(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesProjectSort集合
        List<MesProjectSort> projectSortList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的项目分类代码唯一校验，key：projectSortNo，value：row
        Map<String, String> projectSortNoMap = new HashMap<>();

        //查询质量巡检项目分类表
        List<MesProjectSort> projectSorts = projectSortRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesProjectSort>> projectSortMap = CollectionUtils.isEmpty(projectSorts) ? null :
                projectSorts.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesProjectSort::getProjectSortNo));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //项目分类代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String projectSortNo = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //项目分类名称
                String projectSortName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    projectSortName = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验项目分类代码不重复
                if (!projectSortNoMap.containsKey(projectSortNo)) {
                    projectSortNoMap.put(projectSortNo, String.valueOf(rowNum));
                    //校验 项目分类代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(projectSortMap) && projectSortMap.containsKey(projectSortNo)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + projectSortNoMap.get(projectSortNo) + "行的第A列数据重复;";
                }

                //校验项目分类名称
                if (StringUtils.isEmpty(projectSortName)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }

                //没有错误的时候，封装MesProjectSort
                if (errorNum == 0) {
                    MesProjectSort projectSort;
                    if(isExist){
                        projectSort = projectSortMap.get(projectSortNo).get(0);
                        ConvertBean.serviceModelUpdate(projectSort, userName);
                    }else{
                        projectSort = new MesProjectSort();
                        ConvertBean.serviceModelInitialize(projectSort, userName);
                    }
                    projectSort.setProjectSortNo(projectSortNo);
                    projectSort.setProjectSortName(projectSortName);
                    projectSort.setOrganizeCode(organizeCode);

                    projectSortList.add(projectSort);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : projectSortList);
        return excelImportResultModel;
    }

    /**
     * 质量巡检项目分类扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesProjectSort(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesProjectSort> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesProjectSort projectSort : excelList) {
                    if(StringUtils.isEmpty(projectSort.getId())){
                        projectSortRepository.insert(projectSort);
                    }else{
                        projectSortRepository.save(projectSort);
                    }
                }
            }
        }
    }
}
